/*
 * Help Access Library 
 * A CGI-Program to read Windows Help files in a Web Browser.
 *
 * Copyright (C) 1995-2004 Bernd Herd, http://www.herdsoft.com
 *                                                                  
 * This program is free software; you can redistribute it and/or    
 * modify it under the terms of the GNU General Public License as   
 * published by the Free Software Foundation; either version 2 of   
 * the License.              
 *                                                                  
 * This program is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
 * GNU General Public License for more details.                     
 *                                                                  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software      
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        
 */
/* includes. -- part of WinHelpCGI

   This piece contains functions to handle inclusion of formatting data
   stored in files outside of the .hlp files. 

   This is useful for online publishing purposes.
 
   (C)1996-2004 Herd Software Development, Kettelerstr. 35 / 68642 Buerstadt / Germany"
*/

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include "hlpacces.h"
#include "winhelpcgi.h"

/**
 * Insert an include file to help formatting of auto-created documents.
 */
static
BOOL
_InsertIncludeFile(LPCSTR filename)
{
  BOOL result = FALSE;
  FILE *fin;
  if (NULL!=( fin=fopen(filename, "rt") ))
    {
      int c;
      printf("<!-- Start \"%s\" -->\n", filename);
      while (EOF!=(c=fgetc(fin)))
        putchar(c);

      fclose(fin);
      printf("<!-- End \"%s\" -->\n", filename);

      result = TRUE;
    }

  return result;
}


/**
 * Insert an include file to help formatting of auto-created documents.
 *
 * @param HlpFileName   Filename of the Help file being processed
 * @param section       Section name of the include, for example "body" or "head" etc.
 * @param defaults      Default-String to be included if no include file exists.
 */
void
InsertInclude(LPCSTR HlpFileName, LPCSTR section, LPCSTR defaults)
{
  char language[10];
  char fullname[512];

  ChooseLanguage(language, sizeof(language));

  // helpfile.hlp.section.de.inc
  snprintf(fullname, sizeof(fullname), "%s.%s.%s.inc", HlpFileName, section, language);
  if ( !_InsertIncludeFile(fullname) )
    {
      // helpfile.hlp.section.inc
      snprintf(fullname, sizeof(fullname), "%s.%s.inc", HlpFileName, section);
      
      if ( !_InsertIncludeFile(fullname) )
        {
          // winhelpcgi.cgi.section.de.inc
          snprintf(fullname, sizeof(fullname), "%s.%s.%s.inc", CGI_NAME, section, language);
          
          if ( !_InsertIncludeFile(fullname) )
            {
              // winhelpcgi.cgi.section.inc
              snprintf(fullname, sizeof(fullname), "%s.%s.inc", CGI_NAME, section);
              if ( !_InsertIncludeFile(fullname) && 
                   defaults != NULL)
                {
                  fputs(defaults, stdout);
                }
            }
        }
    }
}


/**
 * Get a suitable language code "de" or "en" from the HTTP_ACCEPT_LANGUAGE
 * environment variable.
 *
 * @param  pLanguage         Pointer to a buffer that will hold the language code on return
 * @param  buffer_size       Size of the buffer in pLanguage
 *
 * @return TRUE if suitable language has been found
 */
BOOL
ChooseLanguage(LPSTR pLanguage, int buffer_size)
{
  const char *accept_language = getenv("HTTP_ACCEPT_LANGUAGE");
  BOOL finished = FALSE;
  while (accept_language && *accept_language && !finished)
    {
      char *dest;
      
      while (*accept_language && strchr(", '", *accept_language))
        {
          accept_language++;
        }

      for (dest=pLanguage; *accept_language && *accept_language!=',' && (dest-pLanguage)<buffer_size-1; )
        {
          *dest++= *accept_language++;
        }
      *dest=0;

      while (*accept_language && strchr(", '", *accept_language))
        {
          accept_language++;
        }

      if (*accept_language==';')
        {
          while (*accept_language && *accept_language!=',')
            {
              accept_language++;
            }
        }

      /* Test for language codes currently supported. */
      if (!strcasecmp(pLanguage, "de") ||
          !strcasecmp(pLanguage, "en"))
        {
          finished = TRUE;
        }
    }

  if (!finished)
    {
      /* Use "en" by default */
      strncmp(pLanguage, "en", buffer_size);
    }

  return finished;
}
