/*
 * Help Access Library 
 * A CGI-Program to read Windows Help files in a Web Browser.
 *
 * Copyright (C) 1995-2004 Bernd Herd, http://www.herdsoft.com
 *                                                                  
 * This program is free software; you can redistribute it and/or    
 * modify it under the terms of the GNU General Public License as   
 * published by the Free Software Foundation; either version 2 of   
 * the License.              
 *                                                                  
 * This program is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
 * GNU General Public License for more details.                     
 *                                                                  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software      
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        
 */
/* winhelpcgi.c -- part of WinHelpCGI

   CGI program used to display Windows Help Files in a web browser
   based on Help Access Library, generates HTML-Output for a Topic as a CGI-Program

   The RTF-Parser contained herein is limited to the subset of RTF-Coding
   applied by Help Access Library. It cannot be used to Convert fully
   featured RTF-Files from Word processing Applications into HTML format.

   This is only an EXAMPLE PROGRAM for Help Access Library and therefore
   it is not a fully featured Help to HTML-Converter, it's just intented do
   give you an Idea of how to deal with Fonts, Colors and Formatting in
   Help Access Library.

   Interesting limitations:
   - Anchors inside of a Topic <a href="...#id"> cannot be handled because
     we'd have to use the HTO_CONTEXT-Flag to do so, and this would drastically
     reduce performance here.
   - SHG-Files built from Metafiles cannot be processed as we'd have to add
     code to convert the Metafile to a Bitmap.
     For Windows, this code is part of our DaVinci Package... For Linux it's not yet available.
   - MAPs/SHGs are not supported.

   (C)1996-2004 Herd Software Development, Kettelerstr. 35 / 68642 Buerstadt / Germany"
*/


#include <windows.h>
#include <fcntl.h>
#if defined(__MSDOS__) || defined(__WIN32__)
#  include <io.h>
#else
#  include <dirent.h>
#endif
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <locale.h>
#include <time.h>
#include <langinfo.h>
#include "hlpacces.h"           // Help Access Library Functions
#include "winhelpcgi.h"         // Header-File to connect the pieces of WinHelpCGI application
#include "shgimpor.h"           // Import of Bitmaps in Help File Format
#include "dib2png.h"            // DIB Image -> PNG file conversion
#include "wmf2png.h"            // Metafile Image -> PNG file conversion


#include <libintl.h>

#define _(String) gettext (String)
#define gettext_noop(String) (String)
#define N_(String) gettext_noop (String)



/* GetQueryVariable --------------------------------------------*
 * Get a Data-Block signifieing the contents of the Form query
 * variable. returns NULL if Variable nit defined.
 *--------------------------------------------------------------*/
LPCSTR GetQueryVariable(LPCSTR QueryString, LPCSTR VarName)
{
  int    VarNameLength= strlen(VarName);
  char * VarContents  = NULL;
  char * dst;
  
  int    cint;
  char   c;
  
  /*---------------- Scan for "KEYWORDS"-Variable ------------------*/
  while (QueryString &&
			memcmp(QueryString, VarName, strlen(VarName))
			)
    {
		if (NULL!=(QueryString = strchr(QueryString, '&')))
		  QueryString++;
    }
  
  /*---------------- If Variable has been found, return contents ---*/
  if (QueryString)
    {  
		VarContents = dst = (LPSTR) malloc(strlen(QueryString));
		
		for (QueryString+=VarNameLength;              /* Step over "KEYWORDS="... */
			  *QueryString&&*QueryString!='&';
			  )
		  {
			 switch (c=(*QueryString++))
				{ 
				case '+': 
				  c=' '; 
				  break;
				case '%': 
				  sscanf(QueryString, "%02x", &cint);
				  c=(char) cint;
				  QueryString+=2;
				}
			 
			 *dst++=c;
		  }
		
		*dst=0;
    }
  
  return VarContents;
}




/* ============= Accessing Bitmaps ============================================*/

/* GetImage
 *
 * Access Contents of an Image
 *
 * Graphics are stored as Internal Files named BMnumber or |BMnumber in
 * the Internal File System. These files have the same structure as MRB and
 * SHG-Files created outside of the Help-Files, but there are two other
 * Compression schemes.
 *
 * The SHGImport routine takes a Memory-Block in this SHG-File format
 * and converts it to a DIB (Device Independend Bitmap) or a Metafile
 * depending on the contents of the File.
 *
 * The Segmentation Parameters Handle defined in the last Parameter of
 * SHGImport has the same structure defined in seg.h first introduced with
 * ThinHelp Version 1.0 and contains the hotspot rectangle and
 * Macro/Jump informations.
 *
 * Things get much more complex when an embedded Window DLL is to be
 * used by the Help File. We don't want to discuss this here...
 */
void GetImage(LPCSTR HlpFileName, LPCSTR HlpImage)
{
  HIFS      ifs = IFSOpen(HlpFileName);
  HIFSFILE  ifsfile;
  DWORD     dwSHGSize;
  HGLOBAL   hgSHG, hDIB=NULL;
  LPSTR     lpSHG;
  HMETAFILE hMetaFile=NULL;
  int       iImageSize;
  METAFILEHEADER metaFileHeader;
  
  /* MIME content-type information */
  fputs( "Content-type: image/png\n\n",stdout);
  
  if (ifs)
	 {
		/* Read the "bm0.." or "|bm0.." - File, which is always in SHG-File format */
		ifsfile = IFSOpenFile(ifs, HlpImage);
		if (ifsfile)
		  {
			 dwSHGSize = IFSSeekFile(ifsfile, 0, SEEK_END);
			 IFSSeekFile(ifsfile, 0, SEEK_SET);
			 
			 hgSHG = GlobalAlloc(GHND, dwSHGSize);
			 lpSHG = (LPSTR) GlobalLock(hgSHG);
			 IFSReadFile(ifsfile, lpSHG, dwSHGSize);
			 
			 /* Convert SHG-File to standard DIB format */
			 SHGImport(
#ifdef _Windows
                    NULL,                           // Parent-Window for error Messages
#endif
						  lpSHG,                          // Input Data Block
						  &hDIB,                          // DIB-Handle
						  &hMetaFile,                     // Metafile-Handle for SHG containing WMF
						  &iImageSize,                    // Will be filled with Informations on placeable Metafiles
                    &metaFileHeader,                // Placeable Metafile header Informations
						  NULL                            // Segmentation Informations
						  );
			 
			 /* Note: SHGs can consist either of Bitmaps or of
				 Metafiles. As we didn't want to make things too
				 complex, we are only supporting Bitmaps in
				 this Application.
				 
				 Bitmaps pasted into Microsoft Word RTF-Code
				 are converted to Metafiles by Microsoft Word and
				 therefore will not be supported by this example
				 Program.
				 
				 To Convert Metafiles to Bitmaps you might like
				 to use our DaVinci package.
			 */
			 
			 GlobalUnlock(hgSHG);
			 GlobalFree(hgSHG);
			 
			 /* for PCs set stdout to binary mode... */
#      ifdef __WIN32__
			 fflush(stdout);
			 setmode( fileno(stdout), O_BINARY);
#      endif
			 
			 if (hDIB)
				{
				  /* Convert the Standard format DIB to a standard PNG image */ 
				  Dib2Png(stdout, hDIB);
				  
				  GlobalFree(hDIB);
				}
          else if (hMetaFile)
            {
#ifdef HAVE_LIBWMF
              Wmf2Png(stdout, hMetaFile, iImageSize, &metaFileHeader);
#endif

#ifndef _Windows
              GlobalFree((HGLOBAL) hMetaFile);
#else
#error "Don't do that..."              
#endif
            }
			 
			 IFSCloseFile(ifsfile);
		  }
		
		
		IFSClose(ifs);
		
	 } else printf( "Content-type: text/html\n\n" ERROR_FONT "Sorry, can't open File %s" NORMAL_FONT, HlpFileName);
}





/* ================ Enumerate Help file Titles ======================= */

#define PAGE_SIZE   500

typedef struct
{ 
  const char *HlpTitleScan;
  const char *HlpFileName;
  int         TitleCounter;
  int         Page;
  char        BaseUrl[500];
  int         PageCount;
  HTOPIC      htopic;
} TITLEDATA, FAR *LPTITLEDATA;



/* TitleEnumerationProcedure
 *
 * Callback-Routine called by EnumerateContents in call to HlpEnumTitles
 */
BOOL FAR PASCAL TitleEnumerationProcedure(LPCSTR lpszTitle, DWORD dwTopicOffset, LPARAM lParam)
{
  LPTITLEDATA lptit = (LPTITLEDATA) lParam;
  const char *HlpTitleScan = lptit->HlpTitleScan;
  const char *p=NULL;
  char *      dup;
  
  if (!HlpTitleScan || !*HlpTitleScan ||
		NULL!=(p=strstr(lpszTitle, HlpTitleScan)))
	 {
		lptit->TitleCounter++;
		
      if (lptit->TitleCounter >= PAGE_SIZE * lptit->Page)
        {
          LPCSTR contextID = HlpTopicFindContextIdForTopicOffset(lptit->htopic, dwTopicOffset);
          if (contextID)
            {
              printf("<LI><a href=\"" CGI_NAME "?FILENAME=%s&CONTEXTID=%s\">", lptit->HlpFileName, UrlEscape(contextID));
            }
          else
            {
              printf("<LI><a href=\"" CGI_NAME "?FILENAME=%s&TOPICOFFSET=0x%08x\">", lptit->HlpFileName, dwTopicOffset);
            }
          
          if (p)
            {
              dup=strdup(lpszTitle);
              dup[p-lpszTitle]=0;
              htmlputs(dup);
              free(dup);
              printf("<b>");
              dup=strdup(p);
              dup[strlen(HlpTitleScan)]=0;
              htmlputs(dup);
              free(dup);
              printf("</b>");
              htmlputs(p+strlen(HlpTitleScan));
            }
          else
            {
              htmlputs(*lpszTitle ? lpszTitle : "<No Title - maybe popup...>");
            }
          
          
          printf("</a>\n");
        }
	 }
  
  return lptit->TitleCounter < (PAGE_SIZE * (lptit->Page+1) );
}


/* DisplayContentsListing
 *
 * Start enumeration of the contents of a help file.
 */
void DisplayContentsListing(HIFS ifs, LPTITLEDATA lptit)
{
  printf("<hr><h2>%s</h2>\n", _("Contents Listing"));
  if (lptit->Page != 0)
    {
      printf("<p><a href=\"" CGI_NAME "?FILENAME=%s&PAGE=%d\">&lt;&lt; %s</a></p>\n", lptit->HlpFileName, lptit->Page-1, _("Back"));
    }
  printf("<ol start=\"%d\">", lptit->Page*PAGE_SIZE + 1);

  
  HlpEnumTitles(ifs, TitleEnumerationProcedure, (LPARAM) lptit);
  
  /* -------- Write a Trailer ------------------------*/
  printf("</ol><hr>");

  if (lptit->TitleCounter < (PAGE_SIZE * (1+lptit->Page) ) )
    {
      puts(_("Contents listing is complete"));
    }
  else
    {
      char url[600];
      snprintf(url, sizeof(url), "<a href=\"" CGI_NAME "?FILENAME=%s&PAGE=%d\">", lptit->HlpFileName, lptit->Page+1);
      printf(_("Maximum Number of Titles in list reached. Click %shere &gt;&gt;</a> for More"), url);
    }
} // DisplayContentsListing


typedef struct
{
  HIFS     ifs;
  HIFSFILE hKWData;
  LPCSTR   HlpFileName;
  int      TopicCounter;
  int      Page;
  HTOPIC   htopic;
} TKWLIST, FAR*LPTKWLIST;


BOOL FAR PASCAL KWEnumProc(LPCSTR lpszKeyWord, DWORD dwDataOffset, UINT uDataCount, LPARAM lParam)
{
  LPTKWLIST tKWList=(LPTKWLIST)lParam;
  int       i;
  DWORD     dwTopicOffset, dwTopicOffsetStart;
  BYTE      abTopicOffset[4];
  char      szTopicTitle[1024];
  
  if (tKWList->TopicCounter >= PAGE_SIZE * tKWList->Page)
    {
      IFSSeekFile(tKWList->hKWData, dwDataOffset, SEEK_SET);
      
      puts("<dt>");
      htmlputs(lpszKeyWord);
      puts("</dt>\n");
      
      for (i=0; i<uDataCount; ++i)
        {
          IFSReadFile(tKWList->hKWData, &abTopicOffset, sizeof(abTopicOffset));
          
          dwTopicOffset = abTopicOffset[0]+(abTopicOffset[1]<<8)+(abTopicOffset[2]<<16)+(abTopicOffset[3]<<24);
          
          if (HlpGetTopicTitle(tKWList->ifs, dwTopicOffset, szTopicTitle, sizeof(szTopicTitle), &dwTopicOffsetStart))
            {
              LPCSTR contextID = HlpTopicFindContextIdForTopicOffset(tKWList->htopic, dwTopicOffset);
              if (contextID)
                {
                  printf("<dd><a href=\"" CGI_NAME "?FILENAME=%s&CONTEXTID=%s\">",
                         tKWList->HlpFileName,
                         UrlEscape(contextID));
                }
              else
                {
                  printf("<dd><a href=\"" CGI_NAME "?FILENAME=%s&TOPICOFFSET=0x%08x\">",
                         tKWList->HlpFileName,
                         dwTopicOffset);
                }
              htmlputs(szTopicTitle);
              puts("</a></dd>");
            }
        }
    }

  return tKWList->TopicCounter++<(tKWList->Page+1) * PAGE_SIZE;
}


/* DisplayKeyWordListing
 *
 * Start enumeration of the contents of a help file.
 */
void DisplayKeyWordListing(HIFS ifs, LPTITLEDATA tit, LPCSTR lpcszKeyWord)
{
  TKWLIST tKWList;
  
  puts("<hr><h2>Keyword Listing</h2><dl>");
  
  tKWList.hKWData     = IFSOpenFile(ifs, "|KWDATA");
  tKWList.ifs         = ifs;
  tKWList.HlpFileName = tit->HlpFileName;
  tKWList.TopicCounter= 0;
  tKWList.Page        = tit->Page;
  tKWList.htopic      = HlpTopicOpen(ifs, /*HTO_FONTS |*/ HTO_CONTEXT/* | HTO_KEYWORDS*/);
  
  if (tKWList.hKWData!=NULL)
	 {
      HlpEnumKeyWords(ifs, KWEnumProc, lpcszKeyWord, "|KWBTREE", (LPARAM)(LPSTR)&tKWList);
      IFSCloseFile(tKWList.hKWData);
	 }
  else
	 printf("Internal File KWDATA not found\n");
  
  HlpTopicClose(tKWList.htopic);

  /* -------- Write a Trailer ------------------------*/
  printf("</dl><hr>");
} // DisplayKeyWordListing




/* EnumerateContents
 *
 * Enumerate the Contents of the Help-File
 *
 * @param HlpFileName       Filename of the .hlp file to be enumerated
 * @param HlpTitleScan      Optional Keywords to be searched in all Titles
 * @param HlpKeyWord        Keyword to be searched in keyword list
 * @param HlpPage           Page number of the list to be shown
 */
void EnumerateContents(LPCSTR HlpFileName, LPCSTR HlpTitleScan, LPCSTR HlpKeyWord, LPCSTR HlpPage)
{
  HIFS            ifs = IFSOpen(HlpFileName);
  TITLEDATA       tit;
  LPHELPFILETITLE lpHelpTitle;

  /* -------- MIME Content-Type ----------------------*/
  printf("Content-type: text/html\n\n");
  
  if (ifs)
	 {
		/* ----------- Get Help file Title -------------------*/
		lpHelpTitle = HlpGetHelpTitle(ifs);
		if (lpHelpTitle)
		  {
			 memset(&tit, 0, sizeof(tit));
			 
			 tit.HlpTitleScan=HlpTitleScan;
			 tit.HlpFileName=HlpFileName;
          tit.Page=HlpPage!=NULL ? atoi(HlpPage) : 0;
          tit.PageCount=(HlpGetTopicCount(ifs) + PAGE_SIZE-1) / PAGE_SIZE;
          tit.htopic = HlpTopicOpen(ifs, /*HTO_FONTS |*/ HTO_CONTEXT/* | HTO_KEYWORDS*/);
			 
			 /* -------- Include Help-File Title as Title -------*/
			 printf("<html><head><title>");
			 htmlputs(lpHelpTitle->HelpTitle ? lpHelpTitle->HelpTitle : HlpFileName);
			 puts("</title>");

          InsertInclude("contents", "head", NULL);
          printf("<link rel=\"up\" href=\"" CGI_NAME "\" title=\"%s\">\n", _("Contents"));
          printf("<link rel=\"bookmark\" href=\"http://www.herdsoft.com/\" title=\"Herd Software Development\">\n");
          printf("<link rel=\"bookmark\" href=\"http://www.herdsoft.com/catalog/ehlp2rtf.html\" title=\"Help to RTF Converter\">\n");
          printf("<link rel=\"bookmark\" href=\"http://www.herdsoft.com/linux/produkte/winhelpcgi.html\" title=\"WinHelpCGI project page\">\n");
          printf("<link rel=\"search\" href=\"" CGI_NAME "?FILENAME=%s&KEYWORD=\" title=\"Glossary\">\n", HlpFileName);

          snprintf(tit.BaseUrl, sizeof(tit.BaseUrl),
                   HlpKeyWord != NULL
                   ? CGI_NAME "?FILENAME=%s&KEYWORD=%s"
                   : CGI_NAME "?FILENAME=%s",
                   HlpFileName, HlpKeyWord);

          printf("<link rel=\"start\" href=\"%s\" title=\"Start\">\n", tit.BaseUrl);
          if (tit.Page>0)
            {
              printf("<link rel=\"prev\" href=\"%s&PAGE=%d\" title=\"%s\">\n", tit.BaseUrl, tit.Page-1, _("Previous Page"));
            }
          if (tit.Page+1<tit.PageCount)
            {
              printf("<link rel=\"next\" href=\"%s&PAGE=%d\" title=\"%s\">\n", tit.BaseUrl, tit.Page+1, _("Next Page"));
            }

          puts("</head>");
          InsertInclude("contents", "body", "<body bgcolor=\"#ffffff\">");
          fputs("<h1>", stdout);
			 htmlputs(lpHelpTitle->HelpTitle ? lpHelpTitle->HelpTitle : HlpFileName);
			 printf("</h1>\n"
                 "<h2>%s</h2>",
                 _("File Informations"));
			 
			 /* -------- Display some Help File informations ----*/
			 printf("<table border=\"1\">"
                 "<tr><td><b>%s:</b></td><td>",
                 _("Filename"));
			 htmlputs(HlpFileName);
			 puts("</td></tr>\n");
			 
			 if (lpHelpTitle->HelpTitle)
				{
				  printf("<tr><td><b>%s:</b></td><td>", _("File Title"));
				  htmlputs(lpHelpTitle->HelpTitle);
				  puts("</td></tr>\n");
				}
			 
			 if (lpHelpTitle->Copyright)
				{
				  printf("<tr><td><b>%s:</b></td><td>", _("Copyright"));
				  htmlputs(lpHelpTitle->Copyright);
				  puts("</td></tr>\n");
				}
			 
			 if (lpHelpTitle->Citation)
				{
				  puts("<tr><td><b>Citation:</b></td><td>");
				  htmlputs(lpHelpTitle->Citation);
				  puts("</td></tr>\n");
				}

          {
            char buffer[500];
            strftime(buffer, sizeof(buffer), nl_langinfo(D_T_FMT), localtime(&lpHelpTitle->GenDate));

            printf("<tr><td><b>%s</b></td><td>%s</td></tr>",
                   _("Date compiled"),
                   buffer);
          }

          printf("<tr><td><b>%s</b></td><td>%d</td></tr>",
                 _("Number of Topics"),
                 HlpGetTopicCount(ifs));
                 
          printf("<tr><td><b>%s</b></td><td>%d</td></tr>",
                 _("Number of Keywords"),
                 HlpGetKeywordCount(ifs));                 

			 if (lpHelpTitle->Citation)
				{
              
				  puts("<tr><td><b>Citation:</b></td><td>");
				  htmlputs(lpHelpTitle->Citation);
				  puts("</td></tr>\n");
				}
			 
			 
			 puts("</table>");
			 
			 /* -------- Allow entering search keys -------------*/
			 printf("<hr><h2>%s</h2>"
					  "<form method=get action=\"" CGI_NAME "\">"
					  "<input name=\"FILENAME\" type=hidden value=\"%s\">"
					  "%s"
					  "<input name=\"TITLESCAN\" size=\"50\" value=\"%s\">"
					  "<INPUT TYPE=SUBMIT value=\"%s\">"
					  "</form>",
                 _("Search functions"),
					  HlpFileName,
                 _("Text to find in title:"),
					  HlpTitleScan?HlpTitleScan:"",
                 _("Start Search")
					  );
			 
			 /* -------- Allow entering search keys -------------*/
			 printf("<form method=get action=\"" CGI_NAME "\">"
					  "<input name=\"FILENAME\" type=hidden value=\"%s\">"
					  "%s"
					  "<input name=\"KEYWORD\" size=50 value=\"%s\">"
					  "<INPUT TYPE=SUBMIT value=\"%s\">"
					  "</form>",
					  HlpFileName,
                 _("List starting with search KeyWord:"),
					  HlpKeyWord?HlpKeyWord:"",
                 _("Start Search")
					  );
			 
			 
			 /* -------- Enumerate the Contents of the Help File-*/
			 if (HlpKeyWord)
				{
				  DisplayKeyWordListing(ifs, &tit, HlpKeyWord);
				}
			 else
            {
              DisplayContentsListing(ifs, &tit);
            }
			 
			 puts("<p><center><font size=2>Created using winhelpcgi &copy; 1996-2004 <a href=\"http://www.herdsoft.com\">Herd Software Development</a></font></center>");

          InsertInclude("contents", "endtext", NULL);

          puts("</body></html>");
			 
			 HlpFreeHelpTitle(lpHelpTitle);
          HlpTopicClose(tit.htopic);
		  }
		else printf(ERROR_FONT "Sorry, can't retrieve Title Data for %s" NORMAL_FONT, HlpFileName);
		
		IFSClose(ifs);
		
	 } else printf(ERROR_FONT "Sorry, can't open File %s" NORMAL_FONT, HlpFileName);
} // EnumerateContents


/* HtmlListHelpDirectoryContents
 *
 * List a Directory containing help files including all the subdirectories
 */
void HtmlListHelpDirectoryContents(LPCSTR lpcszDirName)
{
  
  DIR           * dir=opendir(lpcszDirName);
  struct dirent * ent;
  HIFS           ifs;
  LPHELPFILETITLE tit;
  char            szFullPathName[MAX_PATH];
  LPCSTR          point;
  
  if (dir)
	 {
		while (NULL!=(ent=readdir(dir)))
        {
			 if (strcmp(lpcszDirName, "."))
				{
				  sprintf(szFullPathName, 
							 "%s/%s",
							 lpcszDirName,
							 ent->d_name);
				}
			 else
				{
				  strcpy(szFullPathName, ent->d_name);
				}
			 
			 if (NULL!=(point=strrchr(ent->d_name, '.')) &&
				  (0==  lstrcmpi(point+1, "hlp")||0==lstrcmpi(point+1, "mvb")) &&
				  NULL!=(ifs = IFSOpen(szFullPathName))
				  )
            {
				  if (NULL!=(tit=HlpGetHelpTitle(ifs)))
                {
						printf("<tr><td><a href=\"" CGI_NAME "?FILENAME=%s&TOPICOFFSET=0x%08x\">", szFullPathName, tit->ContentsTopicOffset);
						htmlputs(ent->d_name);
						
						puts("</a></td><td>");
						htmlputs(tit->HelpTitle ? tit->HelpTitle : "-");
						
						puts("</td><td>");
						htmlputs(tit->Copyright ? tit->Copyright : "-");
						
						puts("</td><td>");
						htmlputs(tit->Citation ? tit->Citation : "-");

                  printf("<td align=\"right\"<a href=\"" CGI_NAME "?FILENAME=%s\" title=\"%s\">%d</a></td>", 
                         szFullPathName, 
                         _("Contents Listing"),
                         HlpGetTopicCount(ifs));

                  printf("<td align=\"right\"<a href=\"" CGI_NAME "?FILENAME=%s&KEYWORD=\" title=\"%s\">%d</a></td>", 
                         szFullPathName, 
                         _("Keyword Search"),
                         HlpGetKeywordCount(ifs));

                  {
                    char buffer[500];
                    strftime(buffer, sizeof(buffer), nl_langinfo(D_FMT), localtime(&tit->GenDate));
                    printf("<td>%s</td>", buffer);
                  }
						
						puts("</tr>\n");
						HlpFreeHelpTitle(tit);
                }
				  
				  IFSClose(ifs);
            }
			 else
				if (ent->d_name[0]!='.')
				  {
					 HtmlListHelpDirectoryContents(szFullPathName);
				  }
        }
		
		closedir(dir);
	 } // if (dir)
  
} // HtmlListHelpDirectoryContents


/* HtmlUsage
 *
 * Explain the Usage of this Program as an HTML-Document
 */
void HtmlUsage(void)
{
  /* -------- MIME Content-Type ----------------------*/
  puts("Content-type: text/html\n\n");
  
  printf("<html><head><title>"
         "%s"
         "</title>",
         _("Accessing a Windows Help File"));

  InsertInclude("directory", "head", NULL);

  puts("</head>");

  InsertInclude("directory", "body", "<body bgcolor=\"#ffffff\"");

  printf("<h1>%s</h1>"
         "<p>%s</p>\n"
         "<hr>"
         "<h2>%s</h2>"
         "<form method=GET action=\"" CGI_NAME "\">\n"
         "<p>%s:<INPUT NAME=\"KEYWORD\" size=\"50\" value=\"\"></p>\n"
         "<p><INPUT TYPE=SUBMIT value=\"%s\"></p>"
         "</form>"
         "<hr>"
         
         "<h2>%s</h2>",
         _("Accessing a Windows Help File"),
         _("This CGI-Program allows reading of a Windows Help (WinHelp) file or Media "
           "Viewer 2.0 file from your favorite web Browser."),
         _("Search for a keyword in all available files"),
         _("KeyWord"),
         _("Start Search"),
         _("List of available help files")
		 );

  printf("<table border=1><tr><th>%s</th><th>%s</th><th>%s</th><th>Citation</th><th>%s</th><th>%s</th><th>%s</th></tr>",
         _("Filename"),
         _("File Title"),
         _("Copyright"),
         _("Topics"),
         _("Keywords"),
         _("Date"));

  
  HtmlListHelpDirectoryContents(".");
#ifdef __netware__
  HtmlListHelpDirectoryContents("sys:login");
  HtmlListHelpDirectoryContents("sys:public");
#endif
  
  puts("</table>"
		 "<p>Copyright &copy; 1996-2004 Herd Software Development <a href=\"http://www.herdsoft.com\">http://www.herdsoft.com</a>\n"
		 );
  
  InsertInclude("directory", "endtext", NULL);

  puts("</body></html>");
} // HtmlUsage


/**
 * Check if a given filename can be considered secure, that means it does
 * not point into a directory above the current directory.
 *
 * For OS with unnormal filesystem semantics this must be modified
 */
static
BOOL
_IsSecureFileName(const char * HlpFileName)
{
#ifdef __netware__
  return !strchr(HlpFileName, ':') && !strchr(HlpFileName, '/') && strchr(HlpFileName, '\\');
#else
  /* What a clean concept Posix is compared to Windows...! */
  return HlpFileName[0] != '/' && strstr(HlpFileName, "../")==NULL;
#endif
}



/* ============= Main Program Control =========================================*/


/* WinHelpCGI Main Program
 *
 */
void WinHelpCGI(LPCSTR QueryString)
{
  const char* HlpFileName;
  const char* HlpTopicOffset;
  const char* HlpKeyWord;
  const char* HlpTitleScan;
  const char* HlpImage;
  const char* HlpPage;
  const char* HlpFormat;
  const char* HlpContextId;
  
  HlpFileName    = GetQueryVariable(QueryString, "FILENAME=");
  HlpTopicOffset = GetQueryVariable(QueryString, "TOPICOFFSET=");
  HlpContextId   = GetQueryVariable(QueryString, "CONTEXTID=");
  HlpTitleScan   = GetQueryVariable(QueryString, "TITLESCAN=");
  HlpKeyWord     = GetQueryVariable(QueryString, "KEYWORD=");
  HlpImage       = GetQueryVariable(QueryString, "IMAGE=");
  HlpPage        = GetQueryVariable(QueryString, "PAGE=");
  HlpFormat      = GetQueryVariable(QueryString, "FORMAT=");

  if (HlpFileName &&
      _IsSecureFileName(HlpFileName))
	 {
		if ((HlpTopicOffset && !memcmp(HlpTopicOffset, "0x", 2)) ||
          HlpContextId)
		  EnumerateTopic(HlpFileName, HlpTopicOffset, HlpContextId, HlpFormat);
		else
		  if (HlpImage)
			 GetImage(HlpFileName, HlpImage);
		  else EnumerateContents(HlpFileName, HlpTitleScan, HlpKeyWord, HlpPage);
	 } // else printf(ERROR_FONT "Sorry, Neccessarry Variables are Missing" NORMAL_FONT);
  else
    {
      /* No HelpFile Name specified yet. */
      if (HlpKeyWord)
		  SearchKeyWordInAllHelpFiles(HlpKeyWord);
      else
		  HtmlUsage();
    }
} // WinHelpCGI



/**
 * Set the gettext domain based on the browser language informations.
 */
static
void
_ChooseTextDomain(void)
{
  char language[10];
  if (ChooseLanguage(language, sizeof(language)) )
    {
      if (!strcasecmp(language, "de"))
        {
          setlocale (LC_ALL, "de_DE");
          bindtextdomain (PACKAGE, "locale");
          textdomain (PACKAGE);
        }
    }
}



/**
 * Encode given string in a way so it will always fit into an URL string.
 *
 * @param plain    Plaintext string
 */
const char *UrlEscape(const char *plain)
{
  static char buffer[200];
  char *dest;
  for (dest=buffer; *plain && (dest-buffer)<sizeof(buffer)-4; plain++)
    {
      if ((*plain>='a' && *plain<='z') ||
          (*plain>='A' && *plain<='Z') ||
          (*plain>='0' && *plain<='9') ||
          strchr("._/", *plain)!=NULL
          )
        {
          *dest++=*plain;
        }
      else if (*plain==' ')
        {
          *dest++=' ';
        }
      else
        {
          sprintf(dest, "%%%02X", *plain);
          dest += 3;
        }
    }
  *dest=0;
  return buffer;
}


int
main(int argc, char **argv)
{ 
  const char *QueryString=getenv("QUERY_STRING");

  if (QueryString)
	 { 
      _ChooseTextDomain();

		WinHelpCGI(*QueryString=='\"'?QueryString+1:QueryString);
	 }
  else
	 {
		printf("winhelpcgi: " VERSION " CGI program used to display Windows Help (WinHelp .HLP) files\n"
             "            from HTML pages using a web browser.\n\n"
             "USAGE   :   This Program is intented to be used by a HTTP (WWW-) Server\n"
             "            for Internet Purposes.\n\n"
             "            it is based on the CGI (Common Gateway Interface) standard\n"
             "            for Internet Programming\n\n"
             "            You need to install the " CGI_NAME " program into a CGI-Enabled\n"
             "            directory of your (apache?) web server\n\n"
             "(C)Copyright 1996-2004 Herd Software Development\n"
             "Kettelerstr. 35 / 68642 Buerstadt / Germany / http://www.herdsoft.com\n");
	 }
  
  return 0;
} // main


/*
 * Local variables:
 *  tab-width: 3
 * End:
 */
